c
c Matt.Dobbs@Cern.CH, December 1999
c EXTERNAL statement links PYDATA on most machines.
c (Often necessary for the initialization of Pythia)
c
      subroutine initpydata
      EXTERNAL PYDATA
      return
      end


